#ifndef COLOURS_H
#define COLOURS_H

#include "xbasic_types.h"

/* colours */
/* The upper 8 bits of the colour are used to store the behavior of the pixel
 * under certain operations. The bits are defined as follows:
 * bit 0: fadeable; if this bit is 0, the pixel is not faded by fading operations
 *
 * By default, all colours are created with all upper 8 bits asserted.
 */
typedef Xuint32 colour_t ;

static const int FADE_BIT = 24 ;

static inline colour_t make_colour( colour_t r, colour_t g, colour_t b ) {
	return (0xff << 24) | (b << 16) | (g << 8) | r ;
}

static inline colour_t make_fadeable( colour_t c ) {
	return c | (1 << FADE_BIT) ;
}

static inline colour_t make_unfadeable( colour_t c ) {
	return c & ~(1 << FADE_BIT) ;
}

static const colour_t RED = 0xff0000ff ;
static const colour_t BLUE = 0xffff0000 ;
static const colour_t GREEN = 0xff00ff00 ;
static const colour_t WHITE = 0xffffffff ;
static const colour_t BLACK = 0xff000000 ;

#endif
