#include "xparameters.h"
#include "xbasic_types.h"
#include "xac97.h"
#include "xac97_l.h"
#include "xac97_i.h"
#include "mb_interface.h"
#include "fsl.h"

#include "colours.h"

void set_colour(void *base, Xuint32 x, Xuint32 y, colour_t colour);

/* render module */
#include "render_ops.h"

/* fft controller defines */
const Xuint32 FFT_STATE_OFFSET = 0x4000 ;
const Xuint32 FFT_STATE_IDLE = 2 ;

/* vga controller */
inline void vga_flip_buffer() {
	static volatile Xuint32 *vga = (Xuint32 *) XPAR_OPB_VGA_0_BASEADDR ;
	*vga = 0 ;
	rm_flip_buffer() ;
}

/* push buttons */
#define MODE_SOFTWARE 0
#define MODE_RM_SIMPLE 1
#define MODE_RM_FADE 2
inline int get_visualization_mode() {
	Xuint32 pbVal = *((Xuint32 *)XPAR_OPB_PB_0_BASEADDR);
	if (pbVal & 0x1)
		return MODE_SOFTWARE;
	else if (pbVal & 0x2)
		return MODE_RM_SIMPLE;
	else
		return MODE_RM_FADE;
}

/* program function prototypes */

int triangle_wave( int v, int halfLen ) ;

/* debugging */
void test_vga_zbt_rams() ;

int main (void) 
{
	/*{
		vga_flip_buffer();
		unsigned int *pVga[] = { 
			(unsigned int *) XPAR_OPB_VGA_ZBT_0_BASEADDR,
			(unsigned int *) XPAR_OPB_VGA_ZBT_1_BASEADDR } ;
		Xuint32 i ;
		for( i = 0 ; i < sizeof(pVga)/sizeof(pVga[0]) ; ++i ) {
			Xuint32 x, y ;
			for( x = 0 ; x < 640 ; ++x ) {
				for( y = 0 ; y < 240 ; ++y ) {
					set_colour( pVga[i], x, y, ((x / 4) % 2) ? 0x0000ff : 0xffff00 ) ;
				}
				for( y = 240 ; y < 480 ; ++y ) {
					set_colour( pVga[i], x, y, (x % 2) ? 0x00ffff : 0xff0000 ) ;
				}
			}
			
			vga_flip_buffer();
		}
		
		vga_flip_buffer();
	}*/
	
	
	/*
	{
		Xuint32 i, j;
		unsigned int *pFrameBuffers[] = {
						(unsigned int *) XPAR_OPB_VGA_ZBT_0_BASEADDR,
						(unsigned int *) XPAR_OPB_VGA_ZBT_1_BASEADDR };
		int curBackBuffer;
		
		for (curBackBuffer = 0; curBackBuffer < 2; curBackBuffer++) {
			// initially, buffer 1 is back buffer. since we will be drawing
			// to buffer 0 first, flip buffers.
			vga_flip_buffer();
			set_colour(pFrameBuffers[curBackBuffer], i, 400, BLUE);
			for( j = 0 ; j < 640; j++ ) {
				for( i = 0 ; i < 480; i++ ) {
					if (curBackBuffer) {
						if ( i % 2 == 0 )
							set_colour(pFrameBuffers[curBackBuffer], j, i, RED);
						else
							set_colour(pFrameBuffers[curBackBuffer], j, i, BLUE);
					}
					else {
						if ( j % 4 == 0 || j % 4 == 1 )
							set_colour(pFrameBuffers[curBackBuffer], j, i, GREEN);
						else
							set_colour(pFrameBuffers[curBackBuffer], j, i, WHITE);
					}
				}
			}
		}
		
		for( j = 0 ; j < 640 ; ++j ) {
			for( i = 0 ; i < 480 ; ++i ) {
				if( i == 0 || i == 479 || j == 0 || j == 639 ) {
					set_colour( pFrameBuffers[0], j, i, RED ) ;
					set_colour( pFrameBuffers[1], j, i, BLUE ) ;
				} else if( ((i / 8) % 2) == ((j / 8) % 2)) {
					set_colour( pFrameBuffers[0], j, i, BLUE ) ;
					set_colour( pFrameBuffers[1], j, i, BLACK ) ;
				} else {
					set_colour( pFrameBuffers[0], j, i, BLACK ) ;
					set_colour( pFrameBuffers[1], j, i, BLUE ) ;
				}
			}
		}
		
		vga_flip_buffer();
		vga_flip_buffer();
		vga_flip_buffer();
		vga_flip_buffer();
		
		curBackBuffer = 1 ;
		
		// test clearing background with hardware
		//putfsl(OP_CLEAR_BUFFER, RENDER_MODULE_FSL_ID);
	}*/
	
	/* start the main program */
	{
		volatile Xint32 *xk_real;
		volatile Xint32 *xk_imag;
		
		volatile colour_t *pFrameBuffers[] = {
						(colour_t *) XPAR_OPB_VGA_ZBT_0_BASEADDR,
						(colour_t *) XPAR_OPB_VGA_ZBT_1_BASEADDR };
		int curBackBuffer = 1;
	
		xk_real = (Xint32 *)XPAR_OPB_FFT_CONTROLLER_0_BASEADDR;
		xk_imag = (Xint32 *)(XPAR_OPB_FFT_CONTROLLER_0_BASEADDR + 0x1000);
		
		/* Initialize AC97 Sound controller */
		init_sound(8000);
		record_enable();
		
		/*//vga_flip_buffer();
		for (i = 0; i < 512; i++) {	
			int j;
			for (j = 0; j < 480; j++) {	
					if (j < i)
						set_colour(pFrameBuffers[1], i, j, BLACK);
					else
						set_colour(pFrameBuffers[1], i, j, colours[i]);
				}
		}
		//vga_flip_buffer();
		
		vga_flip_buffer();
		for (i = 0; i < 512; i++) {	
			int j;
			for (j = 0; j < 480; j++) {	
						set_colour(pFrameBuffers[0], i, j, 0x00ff00);
			}
		}
		
		for (i = 0; i < 512; i+=4) {	
			int j;
			for (j = 0; j < 480; j++) {	
					if (j < i)
						set_colour(pFrameBuffers[0], i, j, BLACK);
					else
						set_colour(pFrameBuffers[0], i, j, RED);
				}
		}
		vga_flip_buffer();*/
		
		rm_clear_buffer( BLACK ) ;
		vga_flip_buffer();
		rm_clear_buffer( BLACK ) ;
		vga_flip_buffer();
		
		int curVisualizationMode;
		int redOffset, greenOffset, blueOffset ;
		int redOffsetDir, greenOffsetDir, blueOffsetDir ;
		redOffset = 0 ;
		redOffsetDir = 1 ;
		greenOffset = 0;
		greenOffsetDir = 1 ;
		blueOffset = 0 ;
		blueOffsetDir = 1 ;
		
		int prevHeightAvg = 0 ;
		while (1) {
			int i, j;
			volatile Xuint32 *fft_status = (Xuint32*)(XPAR_OPB_FFT_CONTROLLER_0_BASEADDR
											+ FFT_STATE_OFFSET);
		
			xk_real[0] = 0; /* start FFT */
			
			/* wait for fft to finish */
			while(*fft_status != FFT_STATE_IDLE) {
			}
			
			curVisualizationMode = get_visualization_mode();
			
			/* calculate heights */
			static Xuint32 heights[512] ;
			Xuint32 heightAvg = 0 ;
			
			for( i = 1 ; i < 512 ; i++ ) {
				/* need to sign extend manually */
				Xint32 real = (xk_real[i] << 5) >> 5;
				Xint32 imag = (xk_imag[i] << 5) >> 5;
				Xuint32 energy = real*real + imag*imag;
				Xuint32 height = (energy >> 17);

				heightAvg += height ;
				
				if( height >= 480 )
					height = 479 ;
				
				heights[i] = height ;
			}
			heightAvg /= 511 ;
			
			if( heightAvg > prevHeightAvg ) {
				/* add to average height if the difference compared to previous average
				   height is large */
				int adjustment = (heightAvg - prevHeightAvg) / 15 ;
				heightAvg += (adjustment * adjustment) * (adjustment * adjustment) ;
			}
			prevHeightAvg = heightAvg ;
			
			int colourAvg = 60 + heightAvg * 195 / 80 ;
			
			for (i = 1; i < 512; i++) {				
				Xuint32 height = heights[i] ;
				Xuint32 y_start;
									
				int r, g, b ;
				
				r = colourAvg + triangle_wave(i - redOffset, 75) - 37 ;
				g = colourAvg + triangle_wave(i - greenOffset, 180) / 2 - 45 ;
				b = colourAvg + triangle_wave(i - blueOffset, 160) / 2 - 40 ;
				
				if( r < 0 ) r = 0 ;
				else if( r > 255 ) r = 255 ;
				
				if( g < 0 ) g = 0 ;
				else if( g > 255 ) g = 255 ;
				
				if( b < 0 ) b = 0 ;
				else if( b > 255 ) b = 255 ;
				
				/* draw a vertical line */
				y_start = 479 - height;
				
				/* using render module */
				if (curVisualizationMode == MODE_RM_FADE
						|| curVisualizationMode == MODE_RM_SIMPLE)
				{
					/* 1) Height of bin to draw (0 spans top to bottom, 480 draws nothing)
					 * 2) bin # to draw (i.e. x coordinate)
					 * 3) Colour of bin
					 */			
					rm_draw_fft_bin( y_start, i - 1, make_colour( r, g, b ) ) ;
				}
				/* software rendering */
				else if (curVisualizationMode == MODE_SOFTWARE)
				{
					for (j = 0; j < 480; j++) {
						if (j < y_start)
							set_colour(pFrameBuffers[curBackBuffer], i, j, BLACK);
						else
							set_colour(pFrameBuffers[curBackBuffer], i, j, make_colour( r, g, b ));
					}
				}
			}
			
			// flip back buffers
			vga_flip_buffer();
			
			// clear the new back buffer for drawing
			if (curVisualizationMode == MODE_RM_SIMPLE)
			{
				rm_clear_buffer( BLACK ) ;
			}
			else if (curVisualizationMode == MODE_RM_FADE)
			{
				rm_fade_buffer(0x111111) ;
			}
			curBackBuffer = (curBackBuffer + 1) % 2;
			
			redOffset += redOffsetDir * 10 ;
			greenOffset += greenOffsetDir * 30 ;
			blueOffset += blueOffsetDir * 20 ;
			
			/*if( redOffset < 0 ) redOffset += 525 ;
			else if( redOffset >= 525 ) redOffset -= 525 ;
			
			if( greenOffset < 0 ) greenOffset += 540 ;
			else if( greenOffset >= 540 ) greenOffset -= 540 ;
			
			if( blueOffset < 0 ) blueOffset += 640 ;
			else if( blueOffset >= 640 ) blueOffset -= 640 ;*/
			
			if( redOffset < 0 || redOffset >= 525 ) redOffsetDir *= -1 ;
			if( greenOffset < 0 || greenOffset >= 540 ) greenOffsetDir *= -1 ;
			if( blueOffset < 0 || blueOffset >= 640 ) blueOffsetDir = -1 ;
		}
	}

	exit(0);
}

void set_colour(void *base, Xuint32 x, Xuint32 y, colour_t colour) {
	Xuint32 pixel_offset = y*640 + x; 
	colour_t *pixel = (colour_t *) base + pixel_offset ;
	
	*pixel = colour ;
}

void test_vga_zbt_rams() {
	// ASSUMES current back buffer is #1
	vga_flip_buffer() ;
	
	Xuint32 *pVgaZbt[] = { 
		(Xuint32 *) XPAR_OPB_VGA_ZBT_0_BASEADDR,
		(Xuint32 *) XPAR_OPB_VGA_ZBT_1_BASEADDR,
		(Xuint32 *) XPAR_OPB_VGA_ZBT_2_BASEADDR} ;
	Xuint32 i, j, k ;
	
	for( i = 0 ; i < sizeof(pVgaZbt)/sizeof(pVgaZbt[0]) ; ++i ) {
		Xuint32 *p = pVgaZbt[i] ;
		Xuint32 failureCount = 0 ;
		
		for( j = 0 ; j < 1024*1024*2/4 ; ++j, ++p ) {
			Xuint32 v = 0 ;
			for( k = 0 ; k < 32 ; ++k ) {
				v |= 1 << k ;
				*p = v ;
			
				Xuint32 read = *p ;
				if( read != v ) {
					++failureCount ;
				}
			}
		}
		
		vga_flip_buffer();
	}
	
	// ensure even number of flips occur so that back buffer is back at #1
	// at the end of function
}

int triangle_wave( int v, int halfLen ) {
	int fullLen = halfLen * 2 ;
	
	if( v >= 0 ) {
		int x = v % fullLen ;
		if( x < halfLen )
			return x ;
		else
			return fullLen - x ;
	} else {
		int x = v % fullLen ;
		if( -x < halfLen )
			return -x ;
		else
			return fullLen + x ;
	}
}
