`timescale 1ns / 1ns

// Testbench that connects the opb_vga_zbt controller with
// a behavourial model of the ZBT chip. Top-level simulation entity.
module tb(

) ;
parameter C_OPB_AWIDTH = 32 ;
parameter C_OPB_DWIDTH = 32 ;

// -----------------------------------------------------------------
// port declarations

// OPB signals
reg OPB_Clk ;
reg OPB_Rst ;

// OPB inputs
reg [C_OPB_AWIDTH-1:0] OPB_ABus ;
reg [3:0] OPB_BE ;
reg OPB_RNW ;
reg OPB_select ;
reg OPB_seqAddr ;
reg [C_OPB_DWIDTH-1:0] OPB_DBus ;

// OPB outputs
wire [C_OPB_DWIDTH-1:0] OPB_VGA_ZBT_DBus ;
wire OPB_VGA_ZBT_errAck ;
wire OPB_VGA_ZBT_retry ;
wire OPB_VGA_ZBT_toutSup ;
wire OPB_VGA_ZBT_xferAck ;

// memory inputs
wire [31:0] MIF1_read_data;
reg MIF1_op_done;
wire [31:0] MIF2_read_data;
reg MIF2_op_done;

// memory outputs
wire [31:0] MIF1_addr;
wire MIF1_read;
wire MIF1_write;
wire [31:0] MIF1_write_data;
wire [31:0] MIF2_addr;
wire MIF2_read;
wire MIF2_write;
wire [31:0] MIF2_write_data;

// vga inputs
reg VGA_PixelClockIn;

// vga outputs
wire [7:0] VGA_Red;
wire [7:0] VGA_Green;
wire [7:0] VGA_Blue;
wire VGA_Blank;
wire VGA_Sync;
wire VGA_PixelClockOut;
wire VGA_HSync;
wire VGA_VSync;
				
opb_vga opb_vga_inst(
				// OPB signals
				OPB_Clk,
				OPB_Rst,

				// OPB inputs
				OPB_ABus,
				OPB_BE,
				OPB_RNW,
				OPB_select,
				OPB_seqAddr,
				OPB_DBus,

				// OPB outputs
				OPB_VGA_DBus,
				OPB_VGA_errAck,
				OPB_VGA_retry,
				OPB_VGA_toutSup,
				OPB_VGA_xferAck,

				// VGA signals
				VGA_PixelClockIn,	// should be 25.175 MHz
				VGA_Red,
				VGA_Green,
				VGA_Blue,
				VGA_Blank,
				VGA_Sync,
				VGA_PixelClockOut,
				VGA_HSync,
				VGA_VSync,
				
				// Memory interface to first ZBT module
				MIF1_addr,			// byte address (though bottom-most 2 bits are meaningless)
				MIF1_read,
				MIF1_read_data,
				MIF1_write,
				MIF1_write_data,
				MIF1_op_done,
				
				// Memory interface to second ZBT module
				MIF2_addr,			// byte address (though bottom-most 2 bits are meaningless)
				MIF2_read,
				MIF2_read_data,
				MIF2_write,
				MIF2_write_data,
				MIF2_op_done
) ;


// stimulus

// clocks 
always #20 OPB_Clk = ~OPB_Clk;
always #40 VGA_PixelClockIn = ~VGA_PixelClockIn;

initial begin
	OPB_Clk = 0;
	OPB_Rst = 1;
	OPB_ABus  = 32'b0;
	OPB_BE = 4'b0;
	OPB_RNW = 1'b0;
	OPB_select = 1'b0;
	OPB_seqAddr = 1'b0;
	OPB_DBus = 32'b0;

	VGA_PixelClockIn = 0;

	MIF1_op_done = 0;
	MIF2_op_done = 0;

	# 200
	OPB_Rst = 0;
end

// pixel data
reg [31:0] pixelData;

// memory interface
always @(posedge OPB_Clk) begin
	if (MIF1_read) begin
		#40
		MIF1_op_done = 1;
		#40
		MIF1_op_done = 0;
	end
	else
		MIF1_op_done = 0;
		
	if (MIF2_read) begin
		#40
		MIF2_op_done = 1;
		#40
		MIF2_op_done = 0;
	end
	else
		MIF2_op_done = 0;
end

always @(posedge OPB_Clk or posedge OPB_Rst) begin
	if (OPB_Rst)
		pixelData <= 0;
	else if (MIF1_op_done || MIF2_op_done) begin
		pixelData <= pixelData + 1;
	end
end
assign MIF1_read_data = pixelData;
assign MIF2_read_data = pixelData;

endmodule

