`timescale 1ns / 1ps

// Testbench that connects the opb_vga_zbt controller with
// a behavourial model of the ZBT chip. Top-level simulation entity.
module tb(
				// OPB signals
				OPB_Clk,
				OPB_Rst,

				// OPB inputs
				OPB_ABus,
				OPB_BE,
				OPB_RNW,
				OPB_select,
				OPB_seqAddr,
				OPB_DBus,

				// OPB outputs
				OPB_VGA_DBus,
				OPB_VGA_errAck,
				OPB_VGA_retry,
				OPB_VGA_toutSup,
				OPB_VGA_xferAck,

				VGA_PixelClockIn,

) ;
parameter C_OPB_AWIDTH = 32 ;
parameter C_OPB_DWIDTH = 32 ;

// -----------------------------------------------------------------
// port declarations

// OPB signals
input OPB_Clk ;
input OPB_Rst ;

// OPB inputs
input [C_OPB_AWIDTH-1:0] OPB_ABus ;
input [3:0] OPB_BE ;
input OPB_RNW ;
input OPB_select ;
input OPB_seqAddr ;
input [C_OPB_DWIDTH-1:0] OPB_DBus ;

// OPB outputs
output [C_OPB_DWIDTH-1:0] OPB_VGA_DBus ;
output OPB_VGA_errAck ;
output OPB_VGA_retry ;
output OPB_VGA_toutSup ;
output OPB_VGA_xferAck ;

// memory interface 1
wire [C_OPB_AWIDTH-1:0] MIF1_addr ;
wire MIF1_read ;
wire [C_OPB_DWIDTH-1:0] MIF1_read_data ;
wire MIF1_read_data_ready ;
wire MIF1_write ;
wire [C_OPB_DWIDTH-1:0] MIF1_write_data ;
wire MIF1_write_data_request ;
wire [1:0] MIF1_burst_size ;
wire MIF1_op_done ;

// memory interface 2
wire [C_OPB_AWIDTH-1:0] MIF2_addr ;
wire MIF2_read ;
wire [C_OPB_DWIDTH-1:0] MIF2_read_data ;
wire MIF2_read_data_ready ;
wire MIF2_write ;
wire [C_OPB_DWIDTH-1:0] MIF2_write_data ;
wire MIF2_write_data_request ;
wire [1:0] MIF2_burst_size ;
wire MIF2_op_done ;
	
// vga inputs
input VGA_PixelClockIn;

// vga outputs
wire [7:0] VGA_Red;
wire [7:0] VGA_Green;
wire [7:0] VGA_Blue;
wire VGA_Blank;
wire VGA_Sync;
wire VGA_PixelClockOut;
wire VGA_HSync;
wire VGA_VSync;
				
opb_vga opb_vga_inst(
				// OPB signals
				OPB_Clk,
				OPB_Rst,

				// OPB inputs
				OPB_ABus,
				OPB_BE,
				OPB_RNW,
				OPB_select,
				OPB_seqAddr,
				OPB_DBus,

				// OPB outputs
				OPB_VGA_DBus,
				OPB_VGA_errAck,
				OPB_VGA_retry,
				OPB_VGA_toutSup,
				OPB_VGA_xferAck,

				// VGA signals
				VGA_PixelClockIn,	// should be 25.175 MHz
				VGA_Red,
				VGA_Green,
				VGA_Blue,
				VGA_Blank,
				VGA_Sync,
				VGA_PixelClockOut,
				VGA_HSync,
				VGA_VSync,
				
				// Memory interface to first ZBT module
				MIF1_addr,			// byte address (though bottom-most 2 bits are meaningless)
				MIF1_read,
				MIF1_read_data,
				MIF1_read_data_ready,
				MIF1_write,
				MIF1_write_data,
				MIF1_write_data_request,
				MIF1_burst_size,
				MIF1_op_done,
				
				// Memory interface to second ZBT module
				MIF2_addr,			// byte address (though bottom-most 2 bits are meaningless)
				MIF2_read,
				MIF2_read_data,
				MIF2_read_data_ready,
				MIF2_write,
				MIF2_write_data,
				MIF2_write_data_request,
				MIF2_burst_size,
				MIF2_op_done
) ;


// stimulus

// ZBT ram outputs
wire ZBT1_clk ;
wire ZBT1_clken_n ;
wire ZBT1_we_n ;
wire ZBT1_oe_n ;
wire ZBT1_ce_n ;
wire ZBT1_adv_ld ;
wire ZBT1_wena_n ;
wire ZBT1_wenb_n ;
wire ZBT1_wenc_n ;
wire ZBT1_wend_n ;
wire [18:0] ZBT1_addr ;
wire [31:0] ZBT1_data_I ;
wire [31:0] ZBT1_data_O ;
wire [31:0] ZBT1_data_T ;

wire ZBT1_ce1 ;
wire ZBT1_ce2_n ;
wire ZBT1_lbo ;
wire ZBT1_zz ;

opb_vga_zbt vga_zbt_1(
					// OPB memory interface
					// OPB signals
					.OPB_Clk(OPB_Clk),
					.OPB_Rst(OPB_Rst),

					// OPB inputs
					.OPB_ABus(32'd0),			// byte address
					.OPB_BE(4'd0),
					.OPB_RNW(1'b0),
					.OPB_select(1'b0),
					.OPB_seqAddr(1'b0),
					.OPB_DBus(32'd0),

					// additional memory interface 1
					.MIF1_addr(MIF1_addr),
					.MIF1_be(4'b1111),
					.MIF1_read(MIF1_read),
					.MIF1_read_data(MIF1_read_data),
					.MIF1_read_data_ready(MIF1_read_data_ready),
					.MIF1_write(MIF1_write),
					.MIF1_write_data(MIF1_write_data),
					.MIF1_write_data_request(MIF1_write_data_request),
					.MIF1_burst_size(MIF1_burst_size),
					.MIF1_op_done(MIF1_op_done),

					// additional memory interface 2
					.MIF2_addr(32'd0),
					.MIF2_be(4'd0),
					.MIF2_read(1'b0),
					.MIF2_write(1'b0),
					.MIF2_write_data(32'd0),
					.MIF2_burst_size(2'd0),

					// RAM clock input: clock should be same frequency as OPB
					// clock
					.MEM_Clk(OPB_Clk),

					// ZBT ram outputs
					.ZBT_clk(ZBT1_clk),
					.ZBT_clken_n(ZBT1_clken_n),
					.ZBT_we_n(ZBT1_we_n),
					.ZBT_oe_n(ZBT1_oe_n),
					.ZBT_ce_n(ZBT1_ce_n),
					.ZBT_adv_ld(ZBT1_adv_ld),
					.ZBT_wena_n(ZBT1_wena_n),
					.ZBT_wenb_n(ZBT1_wenb_n),
					.ZBT_wenc_n(ZBT1_wenc_n),
					.ZBT_wend_n(ZBT1_wend_n),
					.ZBT_addr(ZBT1_addr),
					.ZBT_data_I(ZBT1_data_I),
					.ZBT_data_O(ZBT1_data_O),
					.ZBT_data_T(ZBT1_data_T),

					.ZBT_ce1(ZBT1_ce1),
					.ZBT_ce2_n(ZBT1_ce2_n),
					.ZBT_lbo(ZBT1_lbo),
					.ZBT_zz(ZBT1_zz)
				) ;

// ZBT ram outputs
wire ZBT2_clk ;
wire ZBT2_clken_n ;
wire ZBT2_we_n ;
wire ZBT2_oe_n ;
wire ZBT2_ce_n ;
wire ZBT2_adv_ld ;
wire ZBT2_wena_n ;
wire ZBT2_wenb_n ;
wire ZBT2_wenc_n ;
wire ZBT2_wend_n ;
wire [18:0] ZBT2_addr ;
wire [31:0] ZBT2_data_I ;
wire [31:0] ZBT2_data_O ;
wire [31:0] ZBT2_data_T ;

wire ZBT2_ce1 ;
wire ZBT2_ce2_n ;
wire ZBT2_lbo ;
wire ZBT2_zz ;

opb_vga_zbt vga_zbt_2(
					// OPB memory interface
					// OPB signals
					.OPB_Clk(OPB_Clk),
					.OPB_Rst(OPB_Rst),

					// OPB inputs
					.OPB_ABus(32'd0),			// byte address
					.OPB_BE(4'd0),
					.OPB_RNW(1'b0),
					.OPB_select(1'b0),
					.OPB_seqAddr(1'b0),
					.OPB_DBus(32'd0),

					// additional memory interface 1
					.MIF1_addr(MIF2_addr),
					.MIF1_be(4'b1111),
					.MIF1_read(MIF2_read),
					.MIF1_read_data(MIF2_read_data),
					.MIF1_read_data_ready(MIF2_read_data_ready),
					.MIF1_write(MIF2_write),
					.MIF1_write_data(MIF2_write_data),
					.MIF1_write_data_request(MIF2_write_data_request),
					.MIF1_burst_size(MIF2_burst_size),
					.MIF1_op_done(MIF2_op_done),

					// additional memory interface 2
					.MIF2_addr(32'd0),
					.MIF2_be(4'd0),
					.MIF2_read(1'b0),
					.MIF2_write(1'b0),
					.MIF2_write_data(32'd0),
					.MIF2_burst_size(2'd0),

					// RAM clock input: clock should be same frequency as OPB
					// clock
					.MEM_Clk(OPB_Clk),

					// ZBT ram outputs
					.ZBT_clk(ZBT2_clk),
					.ZBT_clken_n(ZBT2_clken_n),
					.ZBT_we_n(ZBT2_we_n),
					.ZBT_oe_n(ZBT2_oe_n),
					.ZBT_ce_n(ZBT2_ce_n),
					.ZBT_adv_ld(ZBT2_adv_ld),
					.ZBT_wena_n(ZBT2_wena_n),
					.ZBT_wenb_n(ZBT2_wenb_n),
					.ZBT_wenc_n(ZBT2_wenc_n),
					.ZBT_wend_n(ZBT2_wend_n),
					.ZBT_addr(ZBT2_addr),
					.ZBT_data_I(ZBT2_data_I),
					.ZBT_data_O(ZBT2_data_O),
					.ZBT_data_T(ZBT2_data_T),

					.ZBT_ce1(ZBT2_ce1),
					.ZBT_ce2_n(ZBT2_ce2_n),
					.ZBT_lbo(ZBT2_lbo),
					.ZBT_zz(ZBT2_zz)
				) ;

wire [31:0] ZBT1_data, ZBT2_data ;

assign ZBT1_data = ZBT1_data_T ? {32{1'bz}} : ZBT1_data_O ;
assign ZBT1_data_I = ZBT1_data ;

assign ZBT2_data = ZBT2_data_T ? {32{1'bz}} : ZBT2_data_O ;
assign ZBT2_data_I = ZBT2_data ;

K7N163601A zbtChip1(
	ZBT1_data,
	ZBT1_addr,
	ZBT1_clk,
	ZBT1_clken_n,
	ZBT1_wena_n,
	ZBT1_wenb_n,
	ZBT1_wenc_n,
	ZBT1_wend_n,
	ZBT1_we_n,
	ZBT1_adv_ld,
	ZBT1_oe_n,
	ZBT1_ce_n,
	ZBT1_ce1,
	ZBT1_ce2_n,
	ZBT1_lbo,
	ZBT1_zz ) ;

K7N163601A zbtChip2(
	ZBT2_data,
	ZBT2_addr,
	ZBT2_clk,
	ZBT2_clken_n,
	ZBT2_wena_n,
	ZBT2_wenb_n,
	ZBT2_wenc_n,
	ZBT2_wend_n,
	ZBT2_we_n,
	ZBT2_adv_ld,
	ZBT2_oe_n,
	ZBT2_ce_n,
	ZBT2_ce1,
	ZBT2_ce2_n,
	ZBT2_lbo,
	ZBT2_zz ) ;

integer i ;
initial begin
	// initialize memory to dummy values
	for( i = 0 ; i < 1024*1024*2/4 ; i = i + 1 ) begin
		zbtChip1.mem[i] = i ;
		zbtChip2.mem[i] = i ;
	end
end

///*
// check VGA outputs
reg [31:0] value, pixValue ;

always @( posedge VGA_PixelClockOut ) begin
	if( opb_vga_inst.horzCount > 144 && opb_vga_inst.horzCount <= 144+640 &&
		opb_vga_inst.vertCount >= 33 && opb_vga_inst.vertCount < 33+480 ) begin
		value = (opb_vga_inst.vertCount - 33) * 640 + (opb_vga_inst.horzCount - 144 - 1) ;
		pixValue = {8'd0, VGA_Blue, VGA_Green, VGA_Red} ;

		
		if( pixValue != value )
			$display( "error at horz=%d vert=%d expected value=%08x pixel value=%08x", opb_vga_inst.horzCount, opb_vga_inst.vertCount,
				value, pixValue ) ;
	end
end
//*/

endmodule

