`timescale 1ns / 100ps

// Arbiter for opb_vga_zbt controller. Arbitrates three
// objects, in priority 1, 2, and lastly 3.
module zbt_arbiter(
					clk,
					rst,

					request1,
					grant1,

					request2,
					grant2,

					request3,
					grant3
) ;
// port declarations
input clk ;
input rst ;

input request1 ;
output grant1 ;

input request2 ;
output grant2 ;

input request3 ;
output grant3 ;

// state machine
localparam STATE_IDLE = 0 ;
localparam STATE_GRANT_1 = 1 ;
localparam STATE_GRANT_2 = 2 ;
localparam STATE_GRANT_3 = 3 ;

reg [1:0] state, next_state ;

reg grant1, grant2, grant3 ;

reg grant1w, grant2w, grant3w ;

always @( posedge clk or posedge rst ) begin
	if( rst )
		state <= STATE_IDLE ;
	else
		state <= next_state ;
end

always @(*) begin
	grant1w = 1'b0 ;
	grant2w = 1'b0 ;
	grant3w = 1'b0 ;

	case( state )
		STATE_IDLE: begin
			casex( {request1, request2, request3} ) 
				3'b1xx: begin
					grant1w = 1'b1 ;
					next_state = STATE_GRANT_1 ;
				end

				3'b01x: begin
					grant2w = 1'b1 ;
					next_state = STATE_GRANT_2 ;
				end

				3'b001: begin
					grant3w = 1'b1 ;
					next_state = STATE_GRANT_3 ;
				end

				3'b000:
					next_state = STATE_IDLE ;

				default:
					next_state = {2{1'bx}} ;
			endcase
		end

		STATE_GRANT_1: begin
			if( request1 ) begin
				grant1w = 1'b1 ;
				next_state = STATE_GRANT_1 ;
			end
			else
				next_state = STATE_IDLE ;
		end

		STATE_GRANT_2: begin
			if( request2 ) begin
				grant2w = 1'b1 ;
				next_state = STATE_GRANT_2 ;
			end
			else
				next_state = STATE_IDLE ;
		end

		STATE_GRANT_3: begin
			if( request3 ) begin
				grant3w = 1'b1 ;
				next_state = STATE_GRANT_3 ;
			end
			else
				next_state = STATE_IDLE ;
		end

		default:
			next_state = {2{1'bx}} ;
	endcase
end

always @( posedge clk ) begin
	grant1 <= grant1w ;
	grant2 <= grant2w ;
	grant3 <= grant3w ;
end
endmodule
