`timescale 1ns / 1ps

// Testbench that connects the opb_vga_zbt controller with
// a behavourial model of the ZBT chip. Top-level simulation entity.
module tb(
					// OPB signals
					OPB_Clk,
					OPB_Rst,

					// OPB inputs
					OPB_ABus,
					OPB_BE,
					OPB_RNW,
					OPB_select,
					OPB_seqAddr,
					OPB_DBus,

					// OPB outputs
					OPB_VGA_ZBT_DBus,
					OPB_VGA_ZBT_errAck,
					OPB_VGA_ZBT_retry,
					OPB_VGA_ZBT_toutSup,
					OPB_VGA_ZBT_xferAck,

					// additional memory interface 1
					MIF1_addr,			// byte address
					MIF1_be,
					MIF1_read,
					MIF1_read_data,
					MIF1_read_data_ready,
					MIF1_write,
					MIF1_write_data,
					MIF1_write_data_request,
					MIF1_burst_size,
					MIF1_op_done,

					// additional memory interface 2
					MIF2_addr,			// byte address
					MIF2_be,
					MIF2_read,
					MIF2_read_data,
					MIF2_read_data_ready,
					MIF2_write,
					MIF2_write_data,
					MIF2_write_data_request,
					MIF2_burst_size,
					MIF2_op_done,

					// RAM clock input: clock should be same frequency as OPB
					// clock
					MEM_Clk
) ;
parameter C_OPB_AWIDTH = 32 ;
parameter C_OPB_DWIDTH = 32 ;

// -----------------------------------------------------------------
// port declarations

// OPB signals
input OPB_Clk ;
input OPB_Rst ;

// OPB inputs
input [C_OPB_AWIDTH-1:0] OPB_ABus ;
input [3:0] OPB_BE ;
input OPB_RNW ;
input OPB_select ;
input OPB_seqAddr ;
input [C_OPB_DWIDTH-1:0] OPB_DBus ;

// OPB outputs
output [C_OPB_DWIDTH-1:0] OPB_VGA_ZBT_DBus ;
output OPB_VGA_ZBT_errAck ;
output OPB_VGA_ZBT_retry ;
output OPB_VGA_ZBT_toutSup ;
output OPB_VGA_ZBT_xferAck ;

// additional memory interface 1
input [C_OPB_AWIDTH-1:0] MIF1_addr ;
input [3:0] MIF1_be ;
input MIF1_read ;
output [C_OPB_DWIDTH-1:0] MIF1_read_data ;
output MIF1_read_data_ready ;
input MIF1_write ;
input [C_OPB_DWIDTH-1:0] MIF1_write_data ;
output MIF1_write_data_request ;
input [1:0] MIF1_burst_size ;
output MIF1_op_done ;

// additional memory interface 2
input [C_OPB_AWIDTH-1:0] MIF2_addr ;
input [3:0] MIF2_be ;
input MIF2_read ;
output [C_OPB_DWIDTH-1:0] MIF2_read_data ;
output MIF2_read_data_ready ;
input MIF2_write ;
input [C_OPB_DWIDTH-1:0] MIF2_write_data ;
output MIF2_write_data_request ;
input [1:0] MIF2_burst_size ;
output MIF2_op_done ;

// memory clock
input MEM_Clk ;

// ZBT ram outputs
wire ZBT_clk ;
wire ZBT_clken_n ;
wire ZBT_we_n ;
wire ZBT_oe_n ;
wire ZBT_ce_n ;
wire ZBT_adv_ld ;
wire ZBT_wena_n ;
wire ZBT_wenb_n ;
wire ZBT_wenc_n ;
wire ZBT_wend_n ;
wire [18:0] ZBT_addr ;
wire [31:0] ZBT_data_I ;
wire [31:0] ZBT_data_O ;
wire [31:0] ZBT_data_T ;

wire ZBT_ce1 ;
wire ZBT_ce2_n ;
wire ZBT_lbo ;
wire ZBT_zz ;

opb_vga_zbt vgaZbt(
					// OPB signals
					OPB_Clk,
					OPB_Rst,

					// OPB inputs
					OPB_ABus,
					OPB_BE,
					OPB_RNW,
					OPB_select,
					OPB_seqAddr,
					OPB_DBus,

					// OPB outputs
					OPB_VGA_ZBT_DBus,
					OPB_VGA_ZBT_errAck,
					OPB_VGA_ZBT_retry,
					OPB_VGA_ZBT_toutSup,
					OPB_VGA_ZBT_xferAck,

					// additional memory interface 1
					MIF1_addr,			// byte address
					MIF1_be,
					MIF1_read,
					MIF1_read_data,
					MIF1_read_data_ready,
					MIF1_write,
					MIF1_write_data,
					MIF1_write_data_request,
					MIF1_burst_size,
					MIF1_op_done,

					// additional memory interface 2
					MIF2_addr,			// byte address
					MIF2_be,
					MIF2_read,
					MIF2_read_data,
					MIF2_read_data_ready,
					MIF2_write,
					MIF2_write_data,
					MIF2_write_data_request,
					MIF2_burst_size,
					MIF2_op_done,

					// RAM clock input: clock should be same frequency as OPB
					// clock
					MEM_Clk,

					// ZBT ram outputs
					ZBT_clk,
					ZBT_clken_n,
					ZBT_we_n,
					ZBT_oe_n,
					ZBT_ce_n,
					ZBT_adv_ld,
					ZBT_wena_n,
					ZBT_wenb_n,
					ZBT_wenc_n,
					ZBT_wend_n,
					ZBT_addr,
					ZBT_data_I,
					ZBT_data_O,
					ZBT_data_T,

					ZBT_ce1,
					ZBT_ce2_n,
					ZBT_lbo,
					ZBT_zz
				) ;	

wire [31:0] ZBT_data ;

assign ZBT_data = ZBT_data_T ? {32{1'bz}} : ZBT_data_O ;

assign ZBT_data_I = ZBT_data ;

K7N163601A zbtChip(
	ZBT_data,
	ZBT_addr,
	ZBT_clk,
	ZBT_clken_n,
	ZBT_wena_n,
	ZBT_wenb_n,
	ZBT_wenc_n,
	ZBT_wend_n,
	ZBT_we_n,
	ZBT_adv_ld,
	ZBT_oe_n,
	ZBT_ce_n,
	ZBT_ce1,
	ZBT_ce2_n,
	ZBT_lbo,
	ZBT_zz ) ;
endmodule

