`timescale 1ns/1ps

// Clears a screen by writing each pixel with black
module clear_buffer(
      Clk,
      Rst,
      Start,
      Done,
      FSL_S_Data,
      FSL_S_Exists,
      FSL_S_Read,
      MIF_addr,
      MIF_be,
      MIF_write,
      MIF_write_data,
	  MIF_burst_size,
      MIF_done
);

input Clk;
input Rst;
input Start;
output Done;
input [31:0] FSL_S_Data;
input FSL_S_Exists;
output FSL_S_Read;
output [31:0] MIF_addr;
output [3:0] MIF_be;
output MIF_write;
output [31:0] MIF_write_data;
output [1:0] MIF_burst_size;
input MIF_done;

// parameters
// NOTE width is assumed to be divisible by 4.
// this assumption is used when setting byte enables.
parameter SCREEN_WIDTH = 640; 
parameter SCREEN_HEIGHT = 480;
parameter SCREEN_MAX_POSITION = (SCREEN_WIDTH*SCREEN_HEIGHT*4)-16;

// states
parameter STATE_IDLE = 3'd0;
parameter STATE_SET_WRITE = 3'd1;
parameter STATE_WRITE_WAIT = 3'd2;
parameter STATE_DONE = 3'd3;
parameter STATE_LOAD_ARGS = 3'd4 ;

// variables
reg [31:0] counter;
reg [31:0] colour ;

// state registers
reg [2:0] cur_state;
reg [2:0] next_state;

always @ (posedge Clk or posedge Rst) begin
   if (Rst)
      cur_state <= STATE_IDLE;
   else
      cur_state <= next_state;
end

// next state logic
always @ ( * ) begin
   case (cur_state)
      STATE_IDLE:
         if (Start)
            next_state <= STATE_LOAD_ARGS;
         else
            next_state <= STATE_IDLE;

	  STATE_LOAD_ARGS:
		  if( FSL_S_Exists )
			  next_state <= STATE_SET_WRITE;
		  else
			  next_state <= STATE_LOAD_ARGS;
            
      // initiate write to memory
      STATE_SET_WRITE:
         next_state <= STATE_WRITE_WAIT;
    
      // wait for write to finish
      STATE_WRITE_WAIT:
         if(MIF_done == 1) begin
             if (counter == SCREEN_MAX_POSITION)
                next_state <= STATE_DONE;
             else
                next_state <= STATE_SET_WRITE;
         end
         else
            next_state <= STATE_WRITE_WAIT;
         
      // full handshake      
      STATE_DONE:
         if (Start)
            next_state <= STATE_DONE;
         else
            next_state <= STATE_IDLE;
      
   endcase
end

// outputs
wire sm_load_arg_0 = (cur_state == STATE_LOAD_ARGS && FSL_S_Exists) ;

/////////////////////
// datapath
always @ (posedge Clk or posedge Rst) begin
   if(Rst)
       counter <= 32'b0;
   else
       if (cur_state == STATE_IDLE)
          counter <= 0;
       else if(cur_state == STATE_WRITE_WAIT && MIF_done == 1)
          counter <= counter + 32'd16;
end
       
always @( posedge Clk ) begin
	if( sm_load_arg_0 )
		colour <= FSL_S_Data ;
end

// module outputs
assign Done = (cur_state == STATE_DONE);
assign FSL_S_Read = sm_load_arg_0 ;
assign MIF_addr = counter;
assign MIF_be = 4'b1111;
assign MIF_write = (cur_state == STATE_SET_WRITE || cur_state == STATE_WRITE_WAIT);
assign MIF_write_data = colour;
assign MIF_burst_size = 2'd3 ;		// burst of 4 words per write
endmodule
