`timescale 1ns / 1ps

// Testbench that connects the opb_vga_zbt controller with
// a behavourial model of the ZBT chip. Top-level simulation entity.
module tb(
				// OPB signals
				OPB_Clk,
				OPB_Rst,

				// OPB inputs
				OPB_ABus,
				OPB_BE,
				OPB_RNW,
				OPB_select,
				OPB_seqAddr,
				OPB_DBus,

				// OPB outputs
				OPB_RM_DBus,
				OPB_RM_errAck,
				OPB_RM_retry,
				OPB_RM_toutSup,
				OPB_RM_xferAck,
			
				// FSL
				FSL_S_Read,
				FSL_S_Data,
				FSL_S_Exists
) ;
parameter C_OPB_AWIDTH = 32 ;
parameter C_OPB_DWIDTH = 32 ;

// -----------------------------------------------------------------
// port declarations

// OPB signals
input OPB_Clk ;
input OPB_Rst ;

// OPB inputs
input [C_OPB_AWIDTH-1:0] OPB_ABus ;
input [3:0] OPB_BE ;
input OPB_RNW ;
input OPB_select ;
input OPB_seqAddr ;
input [C_OPB_DWIDTH-1:0] OPB_DBus ;

// OPB outputs
output [C_OPB_DWIDTH-1:0] OPB_RM_DBus ;
output OPB_RM_errAck ;
output OPB_RM_retry ;
output OPB_RM_toutSup ;
output OPB_RM_xferAck ;

// FSL
output FSL_S_Read;
input [31:0] FSL_S_Data;
input FSL_S_Exists;

// memory interface 1
wire [C_OPB_AWIDTH-1:0] MIF1_addr ;
wire [3:0] MIF1_be ;
wire MIF1_read ;
wire [C_OPB_DWIDTH-1:0] MIF1_read_data ;
wire MIF1_read_data_ready ;
wire MIF1_write ;
wire [C_OPB_DWIDTH-1:0] MIF1_write_data ;
wire MIF1_write_data_request ;
wire [1:0] MIF1_burst_size ;
wire MIF1_op_done ;

// memory interface 2
wire [C_OPB_AWIDTH-1:0] MIF2_addr ;
wire [3:0] MIF2_be ;
wire MIF2_read ;
wire [C_OPB_DWIDTH-1:0] MIF2_read_data ;
wire MIF2_read_data_ready ;
wire MIF2_write ;
wire [C_OPB_DWIDTH-1:0] MIF2_write_data ;
wire MIF2_write_data_request ;
wire [1:0] MIF2_burst_size ;
wire MIF2_op_done ;

// memory interface mirror
wire [C_OPB_AWIDTH-1:0] MIF_MIRROR_addr ;
wire [3:0] MIF_MIRROR_be ;
wire MIF_MIRROR_read ;
wire [C_OPB_DWIDTH-1:0] MIF_MIRROR_read_data ;
wire MIF_MIRROR_read_data_ready ;
wire MIF_MIRROR_write ;
wire [C_OPB_DWIDTH-1:0] MIF_MIRROR_write_data ;
wire MIF_MIRROR_write_data_request ;
wire [1:0] MIF_MIRROR_burst_size ;
wire MIF_MIRROR_op_done ;
	
render_module render_module_inst(
				// OPB signals
				OPB_Clk,
				OPB_Rst,

				// OPB inputs
				OPB_ABus,
				OPB_BE,
				OPB_RNW,
				OPB_select,
				OPB_seqAddr,
				OPB_DBus,

				// OPB outputs
				OPB_RM_DBus,
				OPB_RM_errAck,
				OPB_RM_retry,
				OPB_RM_toutSup,
				OPB_RM_xferAck,
			
				// FSL
				FSL_S_Read,
				FSL_S_Data,
				FSL_S_Exists,

				// Memory interface to first ZBT module
				MIF1_addr,			// byte address (though bottom-most 2 bits are meaningless)
				MIF1_be,
				MIF1_read,
				MIF1_read_data,
				MIF1_read_data_ready,
				MIF1_write,
				MIF1_write_data,
				MIF1_write_data_request,
				MIF1_burst_size,
				MIF1_op_done,
				
				// Memory interface to second ZBT module
				MIF2_addr,			// byte address (though bottom-most 2 bits are meaningless)
				MIF2_be,
				MIF2_read,
				MIF2_read_data,
				MIF2_read_data_ready,
				MIF2_write,
				MIF2_write_data,
				MIF2_write_data_request,
				MIF2_burst_size,
				MIF2_op_done,

				// Memory interface to mirror ZBT module
				MIF_MIRROR_addr,			// byte address (though bottom-most 2 bits are meaningless)
				MIF_MIRROR_be,
				MIF_MIRROR_read,
				MIF_MIRROR_read_data,
				MIF_MIRROR_read_data_ready,
				MIF_MIRROR_write,
				MIF_MIRROR_write_data,
				MIF_MIRROR_write_data_request,
				MIF_MIRROR_burst_size,
				MIF_MIRROR_op_done
) ;


// stimulus

// ZBT ram outputs
wire ZBT1_clk ;
wire ZBT1_clken_n ;
wire ZBT1_we_n ;
wire ZBT1_oe_n ;
wire ZBT1_ce_n ;
wire ZBT1_adv_ld ;
wire ZBT1_wena_n ;
wire ZBT1_wenb_n ;
wire ZBT1_wenc_n ;
wire ZBT1_wend_n ;
wire [18:0] ZBT1_addr ;
wire [31:0] ZBT1_data_I ;
wire [31:0] ZBT1_data_O ;
wire [31:0] ZBT1_data_T ;

wire ZBT1_ce1 ;
wire ZBT1_ce2_n ;
wire ZBT1_lbo ;
wire ZBT1_zz ;

opb_vga_zbt vga_zbt_1(
					// OPB memory interface
					// OPB signals
					.OPB_Clk(OPB_Clk),
					.OPB_Rst(OPB_Rst),

					// OPB inputs
					.OPB_ABus(32'd0),			// byte address
					.OPB_BE(4'd0),
					.OPB_RNW(1'b0),
					.OPB_select(1'b0),
					.OPB_seqAddr(1'b0),
					.OPB_DBus(32'd0),

					// additional memory interface 1
					.MIF2_addr(MIF1_addr),
					.MIF2_be(MIF1_be),
					.MIF2_read(MIF1_read),
					.MIF2_read_data(MIF1_read_data),
					.MIF2_read_data_ready(MIF1_read_data_ready),
					.MIF2_write(MIF1_write),
					.MIF2_write_data(MIF1_write_data),
					.MIF2_write_data_request(MIF1_write_data_request),
					.MIF2_burst_size(MIF1_burst_size),
					.MIF2_op_done(MIF1_op_done),

					// additional memory interface 2
					.MIF1_addr(32'd0),
					.MIF1_be(4'd0),
					.MIF1_read(1'b0),
					.MIF1_write(1'b0),
					.MIF1_write_data(32'd0),
					.MIF1_burst_size(2'd0),

					// RAM clock input: clock should be same frequency as OPB
					// clock
					.MEM_Clk(OPB_Clk),

					// ZBT ram outputs
					.ZBT_clk(ZBT1_clk),
					.ZBT_clken_n(ZBT1_clken_n),
					.ZBT_we_n(ZBT1_we_n),
					.ZBT_oe_n(ZBT1_oe_n),
					.ZBT_ce_n(ZBT1_ce_n),
					.ZBT_adv_ld(ZBT1_adv_ld),
					.ZBT_wena_n(ZBT1_wena_n),
					.ZBT_wenb_n(ZBT1_wenb_n),
					.ZBT_wenc_n(ZBT1_wenc_n),
					.ZBT_wend_n(ZBT1_wend_n),
					.ZBT_addr(ZBT1_addr),
					.ZBT_data_I(ZBT1_data_I),
					.ZBT_data_O(ZBT1_data_O),
					.ZBT_data_T(ZBT1_data_T),

					.ZBT_ce1(ZBT1_ce1),
					.ZBT_ce2_n(ZBT1_ce2_n),
					.ZBT_lbo(ZBT1_lbo),
					.ZBT_zz(ZBT1_zz)
				) ;

// ZBT ram outputs
wire ZBT2_clk ;
wire ZBT2_clken_n ;
wire ZBT2_we_n ;
wire ZBT2_oe_n ;
wire ZBT2_ce_n ;
wire ZBT2_adv_ld ;
wire ZBT2_wena_n ;
wire ZBT2_wenb_n ;
wire ZBT2_wenc_n ;
wire ZBT2_wend_n ;
wire [18:0] ZBT2_addr ;
wire [31:0] ZBT2_data_I ;
wire [31:0] ZBT2_data_O ;
wire [31:0] ZBT2_data_T ;

wire ZBT2_ce1 ;
wire ZBT2_ce2_n ;
wire ZBT2_lbo ;
wire ZBT2_zz ;

opb_vga_zbt vga_zbt_2(
					// OPB memory interface
					// OPB signals
					.OPB_Clk(OPB_Clk),
					.OPB_Rst(OPB_Rst),

					// OPB inputs
					.OPB_ABus(32'd0),			// byte address
					.OPB_BE(4'd0),
					.OPB_RNW(1'b0),
					.OPB_select(1'b0),
					.OPB_seqAddr(1'b0),
					.OPB_DBus(32'd0),

					// additional memory interface 1
					.MIF2_addr(MIF2_addr),
					.MIF2_be(MIF2_be),
					.MIF2_read(MIF2_read),
					.MIF2_read_data(MIF2_read_data),
					.MIF2_read_data_ready(MIF2_read_data_ready),
					.MIF2_write(MIF2_write),
					.MIF2_write_data(MIF2_write_data),
					.MIF2_write_data_request(MIF2_write_data_request),
					.MIF2_burst_size(MIF2_burst_size),
					.MIF2_op_done(MIF2_op_done),

					// additional memory interface 2
					.MIF1_addr(32'd0),
					.MIF1_be(4'd0),
					.MIF1_read(1'b0),
					.MIF1_write(1'b0),
					.MIF1_write_data(32'd0),
					.MIF1_burst_size(2'd0),

					// RAM clock input: clock should be same frequency as OPB
					// clock
					.MEM_Clk(OPB_Clk),

					// ZBT ram outputs
					.ZBT_clk(ZBT2_clk),
					.ZBT_clken_n(ZBT2_clken_n),
					.ZBT_we_n(ZBT2_we_n),
					.ZBT_oe_n(ZBT2_oe_n),
					.ZBT_ce_n(ZBT2_ce_n),
					.ZBT_adv_ld(ZBT2_adv_ld),
					.ZBT_wena_n(ZBT2_wena_n),
					.ZBT_wenb_n(ZBT2_wenb_n),
					.ZBT_wenc_n(ZBT2_wenc_n),
					.ZBT_wend_n(ZBT2_wend_n),
					.ZBT_addr(ZBT2_addr),
					.ZBT_data_I(ZBT2_data_I),
					.ZBT_data_O(ZBT2_data_O),
					.ZBT_data_T(ZBT2_data_T),

					.ZBT_ce1(ZBT2_ce1),
					.ZBT_ce2_n(ZBT2_ce2_n),
					.ZBT_lbo(ZBT2_lbo),
					.ZBT_zz(ZBT2_zz)
				) ;

// ZBT ram outputs
wire ZBT3_clk ;
wire ZBT3_clken_n ;
wire ZBT3_we_n ;
wire ZBT3_oe_n ;
wire ZBT3_ce_n ;
wire ZBT3_adv_ld ;
wire ZBT3_wena_n ;
wire ZBT3_wenb_n ;
wire ZBT3_wenc_n ;
wire ZBT3_wend_n ;
wire [18:0] ZBT3_addr ;
wire [31:0] ZBT3_data_I ;
wire [31:0] ZBT3_data_O ;
wire [31:0] ZBT3_data_T ;

wire ZBT3_ce1 ;
wire ZBT3_ce2_n ;
wire ZBT3_lbo ;
wire ZBT3_zz ;

opb_vga_zbt vga_zbt_3(
					// OPB memory interface
					// OPB signals
					.OPB_Clk(OPB_Clk),
					.OPB_Rst(OPB_Rst),

					// OPB inputs
					.OPB_ABus(32'd0),			// byte address
					.OPB_BE(4'd0),
					.OPB_RNW(1'b0),
					.OPB_select(1'b0),
					.OPB_seqAddr(1'b0),
					.OPB_DBus(32'd0),

					// additional memory interface 1
					.MIF2_addr(MIF_MIRROR_addr),
					.MIF2_be(MIF_MIRROR_be),
					.MIF2_read(MIF_MIRROR_read),
					.MIF2_read_data(MIF_MIRROR_read_data),
					.MIF2_read_data_ready(MIF_MIRROR_read_data_ready),
					.MIF2_write(MIF_MIRROR_write),
					.MIF2_write_data(MIF_MIRROR_write_data),
					.MIF2_write_data_request(MIF_MIRROR_write_data_request),
					.MIF2_burst_size(MIF_MIRROR_burst_size),
					.MIF2_op_done(MIF_MIRROR_op_done),

					// additional memory interface 2
					.MIF1_addr(32'd0),
					.MIF1_be(4'd0),
					.MIF1_read(1'b0),
					.MIF1_write(1'b0),
					.MIF1_write_data(32'd0),
					.MIF1_burst_size(2'd0),

					// RAM clock input: clock should be same frequency as OPB
					// clock
					.MEM_Clk(OPB_Clk),

					// ZBT ram outputs
					.ZBT_clk(ZBT3_clk),
					.ZBT_clken_n(ZBT3_clken_n),
					.ZBT_we_n(ZBT3_we_n),
					.ZBT_oe_n(ZBT3_oe_n),
					.ZBT_ce_n(ZBT3_ce_n),
					.ZBT_adv_ld(ZBT3_adv_ld),
					.ZBT_wena_n(ZBT3_wena_n),
					.ZBT_wenb_n(ZBT3_wenb_n),
					.ZBT_wenc_n(ZBT3_wenc_n),
					.ZBT_wend_n(ZBT3_wend_n),
					.ZBT_addr(ZBT3_addr),
					.ZBT_data_I(ZBT3_data_I),
					.ZBT_data_O(ZBT3_data_O),
					.ZBT_data_T(ZBT3_data_T),

					.ZBT_ce1(ZBT3_ce1),
					.ZBT_ce2_n(ZBT3_ce2_n),
					.ZBT_lbo(ZBT3_lbo),
					.ZBT_zz(ZBT3_zz)
				) ;

wire [31:0] ZBT1_data, ZBT2_data, ZBT3_data ;

assign ZBT1_data = ZBT1_data_T ? {32{1'bz}} : ZBT1_data_O ;
assign ZBT1_data_I = ZBT1_data ;

assign ZBT2_data = ZBT2_data_T ? {32{1'bz}} : ZBT2_data_O ;
assign ZBT2_data_I = ZBT2_data ;

assign ZBT3_data = ZBT3_data_T ? {32{1'bz}} : ZBT3_data_O ;
assign ZBT3_data_I = ZBT3_data ;

K7N163601A zbtChip1(
	ZBT1_data,
	ZBT1_addr,
	ZBT1_clk,
	ZBT1_clken_n,
	ZBT1_wena_n,
	ZBT1_wenb_n,
	ZBT1_wenc_n,
	ZBT1_wend_n,
	ZBT1_we_n,
	ZBT1_adv_ld,
	ZBT1_oe_n,
	ZBT1_ce_n,
	ZBT1_ce1,
	ZBT1_ce2_n,
	ZBT1_lbo,
	ZBT1_zz ) ;

K7N163601A zbtChip2(
	ZBT2_data,
	ZBT2_addr,
	ZBT2_clk,
	ZBT2_clken_n,
	ZBT2_wena_n,
	ZBT2_wenb_n,
	ZBT2_wenc_n,
	ZBT2_wend_n,
	ZBT2_we_n,
	ZBT2_adv_ld,
	ZBT2_oe_n,
	ZBT2_ce_n,
	ZBT2_ce1,
	ZBT2_ce2_n,
	ZBT2_lbo,
	ZBT2_zz ) ;

K7N163601A zbtChip3(
	ZBT3_data,
	ZBT3_addr,
	ZBT3_clk,
	ZBT3_clken_n,
	ZBT3_wena_n,
	ZBT3_wenb_n,
	ZBT3_wenc_n,
	ZBT3_wend_n,
	ZBT3_we_n,
	ZBT3_adv_ld,
	ZBT3_oe_n,
	ZBT3_ce_n,
	ZBT3_ce1,
	ZBT3_ce2_n,
	ZBT3_lbo,
	ZBT3_zz ) ;

integer i ;
initial begin
	// initialize memory to dummy values
	for( i = 0 ; i < 1024*1024*2/4 ; i = i + 1 ) begin
		zbtChip1.mem[i] = i ;
		zbtChip2.mem[i] = i ;
		zbtChip3.mem[i] = i ;
	end
end



endmodule

